'use strict';

exports.__esModule = true;
exports.getRules = exports.buildMap = exports.getFieldTypes = exports.FIELD_TYPES = exports.FORM_LAYOUT = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _antd = require('antd');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TextArea = _antd.Input.TextArea;

var Option = _antd.Select.Option;
var CheckboxGroup = _antd.Checkbox.Group;
var RadioGroup = _antd.Radio.Group;
var RadioButton = _antd.Radio.Button;
var InputGroup = _antd.Input.Group;
var MonthPicker = _antd.DatePicker.MonthPicker,
    RangePicker = _antd.DatePicker.RangePicker,
    WeekPicker = _antd.DatePicker.WeekPicker;
var FORM_LAYOUT = exports.FORM_LAYOUT = {
    horizontal: "horizontal",
    vertical: "vertical",
    inline: "inline"
};

var selectOptions = function selectOptions(options) {
    var view = [];
    options.map(function (obj) {
        return view.push(_react2.default.createElement(
            Option,
            { key: obj.value, value: obj.value },
            obj.text
        ));
    });
    return view;
};

var radioItems = function radioItems(items, type) {
    var view = [];
    type === "radio" ? items.map(function (obj) {
        return view.push(_react2.default.createElement(
            _antd.Radio,
            { key: obj.value, value: obj.value },
            obj.text
        ));
    }) : items.map(function (obj) {
        return view.push(_react2.default.createElement(
            RadioButton,
            { key: obj.value, value: obj.value },
            obj.text
        ));
    });
    return view;
};

var FIELD_TYPES = exports.FIELD_TYPES = {
    AutoComplete: 'AutoComplete',
    Cascader: 'Cascader',
    Checkbox: 'Checkbox',
    CheckboxGroup: 'CheckboxGroup',
    DatePicker: 'DatePicker',
    MonthPicker: 'MonthPicker',
    RangePicker: 'RangePicker',
    WeekPicker: 'WeekPicker',
    Input: 'Input',
    TextArea: 'TextArea',
    InputGroup: 'InputGroup',
    InputNumber: 'InputNumber',
    Mention: 'Mention',
    Rate: 'Rate',
    Radio: 'Radio',
    RadioButton: 'RadioButton',
    Select: 'Select',
    Slider: 'Slider',
    Switch: 'Switch',
    TreeSelect: 'TreeSelect',
    TimePicker: 'TimePicker',
    Transfer: 'Transfer',
    Upload: 'Upload'
};

var getFieldTypes = exports.getFieldTypes = function getFieldTypes(fieldProps, options) {
    return {
        'AutoComplete': _react2.default.createElement(_antd.AutoComplete, _extends({}, fieldProps, { filterOption: function filterOption(inputValue, option) {
                return option.props.children.toUpperCase().indexOf(inputValue.toUpperCase()) !== -1;
            } })),
        'Cascader': _react2.default.createElement(_antd.Cascader, fieldProps),
        'Checkbox': _react2.default.createElement(_antd.Checkbox, fieldProps),
        'CheckboxGroup': _react2.default.createElement(CheckboxGroup, fieldProps),
        'DatePicker': _react2.default.createElement(_antd.DatePicker, fieldProps),
        'MonthPicker': _react2.default.createElement(MonthPicker, fieldProps),
        'RangePicker': _react2.default.createElement(RangePicker, fieldProps),
        'WeekPicker': _react2.default.createElement(WeekPicker, fieldProps),
        'Input': _react2.default.createElement(_antd.Input, fieldProps),
        'TextArea': _react2.default.createElement(TextArea, fieldProps),
        'InputGroup': _react2.default.createElement(InputGroup, fieldProps),
        'InputNumber': _react2.default.createElement(_antd.InputNumber, fieldProps),
        'Mention': _react2.default.createElement(_antd.Mention, fieldProps),
        'Rate': _react2.default.createElement(_antd.Rate, fieldProps),
        'Radio': _react2.default.createElement(
            RadioGroup,
            fieldProps,
            options !== undefined ? radioItems(options, 'radio') : null
        ),
        'RadioButton': _react2.default.createElement(
            RadioGroup,
            fieldProps,
            options !== undefined ? radioItems(options, 'button') : null
        ),
        'Select': _react2.default.createElement(
            _antd.Select,
            fieldProps,
            options !== undefined ? selectOptions(options) : null
        ),
        'Slider': _react2.default.createElement(_antd.Slider, fieldProps),
        'Switch': _react2.default.createElement(_antd.Switch, fieldProps),
        'TreeSelect': _react2.default.createElement(_antd.TreeSelect, fieldProps),
        'TimePicker': _react2.default.createElement(_antd.TimePicker, fieldProps),
        'Transfer': _react2.default.createElement(_antd.Transfer, fieldProps),
        'Upload': _react2.default.createElement(_antd.Upload, fieldProps)
    };
};

var RULE_TYPE = {
    string: "string",
    number: "number",
    boolean: "boolean",
    method: "method",
    regexp: "regexp",
    integer: "integer",
    float: "float",
    array: "array",
    object: "object",
    enum: "enum",
    date: "date",
    url: "url",
    hex: "hex",
    email: "email"
};

var buildMap = exports.buildMap = function buildMap(obj) {
    var map = new Map();
    Object.keys(obj).forEach(function (key) {
        map.set(key, obj[key]);
    });
    return map;
};

var getRules = exports.getRules = function getRules(formRule, fieldName) {
    var ruleArray = [];
    var fieldRule = formRule.get(fieldName);

    var _loop = function _loop(rule) {
        var ruleBody = fieldRule[rule];
        var rulePart = ruleBody.split(',');
        // console.log(rulePart)
        switch (rulePart[0]) {
            case 'req':
                ruleArray.push({
                    required: true,
                    message: 'This is 必须的'
                });
                break;

            case 'str':
                ruleArray.push({
                    min: Number(rulePart[1]),
                    max: Number(rulePart[2]),
                    message: ''
                });
                break;

            case 'int':
                ruleArray.push({
                    transform: function transform(value) {
                        return Number(value);
                    },
                    min: Number(rulePart[1]),
                    max: Number(rulePart[2]),
                    type: RULE_TYPE.integer,
                    message: ''
                });
                break;

            case 'float':
                ruleArray.push({
                    min: Number(rulePart[1]),
                    max: Number(rulePart[1]),
                    type: 'float',
                    message: ''
                });
                break;

            case 'date':
                ruleArray.push({
                    transform: function transform(value) {
                        return new Date(value);
                    },
                    type: 'date',
                    message: ''
                });
                break;

            case 'time':
                ruleArray.push({
                    transform: function transform(value) {
                        return new Date(value);
                    },
                    type: 'date',
                    message: ''
                });
                break;
            case 'bool':
                ruleArray.push({
                    type: 'boolean',
                    message: ''
                });
                break;

            case 'email':
                ruleArray.push({
                    type: 'email',
                    message: ''
                });
                break;

            case 'url':
                ruleArray.push({
                    type: 'url',
                    message: ''
                });
                break;

            case 'in':
                ruleArray.push({
                    message: ''
                }, {
                    validator: function validator(rule, value, callback) {
                        rulePart.shift();
                        rulePart.includes(value) ? callback() : callback('Not In Range');
                    }
                });
                break;

            default:
                break;
        }
    };

    for (var rule in fieldRule) {
        _loop(rule);
    }
    return ruleArray;
};