'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _antd = require('antd');

var _Loading = require('./Loading');

var _Loading2 = _interopRequireDefault(_Loading);

var _axiosClient = require('axios-client');

var _fieldConfig = require('./fieldConfig');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FormItem = _antd.Form.Item;

var renderRow = function renderRow(props, formFields, fieldsValue, formRule, isExpand) {
    var renderingRow = [];
    var uuid = 0;

    for (var fieldName in formFields) {
        uuid++;
        var fieldItem = formFields[fieldName];

        if (fieldName.includes('ROW')) {
            var renderingRowFields = [];
            var uuuid = 0;

            for (var rowFieldName in fieldItem.fields) {
                uuuid++;
                var rowFieldItem = fieldItem.fields[rowFieldName];
                renderingRowFields.push(renderField(props, rowFieldName, rowFieldItem, fieldsValue, formRule));
            }
            renderingRow.push(_react2.default.createElement(
                _antd.Row,
                { style: { display: isExpand ? 'none' : 'block' }, gutter: fieldItem.gutter, key: 'row' + uuid },
                renderingRowFields
            ));
        } else if (fieldName.includes('DIVIDER')) {
            var divider = fieldItem.text !== undefined ? _react2.default.createElement(
                _antd.Divider,
                _extends({ key: fieldName }, fieldItem),
                fieldItem.text
            ) : _react2.default.createElement(_antd.Divider, _extends({ key: 'row' + fieldName }, fieldItem));
            renderingRow.push(_react2.default.createElement(
                _antd.Row,
                { style: { display: isExpand ? 'none' : 'block' }, gutter: 24, key: 'row' + uuid },
                divider
            ));
        }
    }
    return renderingRow;
};

var renderField = function renderField(props, fieldName, fieldItem, fieldsValue, formRule, isExpand) {
    var getFieldDecorator = props.form.getFieldDecorator;

    var formItemLayout = props.formLayout === _fieldConfig.FORM_LAYOUT.horizontal ? {
        labelCol: { span: 8 },
        wrapperCol: { span: 16 }
    } : props.formLayout === _fieldConfig.FORM_LAYOUT.vertical ? { colon: true } : null;

    var renderingFields = [];

    var initialValue = fieldsValue.get(fieldName);
    var getFieldDecoratorOptions = {
        initialValue: initialValue,
        rules: (0, _fieldConfig.getRules)(formRule, fieldName)
    };

    getFieldDecoratorOptions.valuePropName = ['Checkbox', 'Switch'].includes(fieldItem.types) ? 'checked' : 'value';

    if (fieldItem.types !== undefined && typeof fieldItem.types !== 'function') {
        if (props.formLayout === _fieldConfig.FORM_LAYOUT.inline || props.formColumn === 1) {
            renderingFields.push(_react2.default.createElement(
                FormItem,
                _extends({}, formItemLayout, {
                    label: fieldItem.label,
                    key: fieldName,
                    style: { display: isExpand ? 'none' : 'block' }
                }),
                getFieldDecorator(fieldName, _extends({}, getFieldDecoratorOptions))((0, _fieldConfig.getFieldTypes)(fieldItem.fieldProps, fieldItem.options)[fieldItem.types])
            ));
        } else {
            renderingFields.push(_react2.default.createElement(
                _antd.Col,
                _extends({}, fieldItem.column, { key: fieldName, style: { display: isExpand ? 'none' : 'block' } }),
                _react2.default.createElement(
                    FormItem,
                    _extends({}, formItemLayout, {
                        label: fieldItem.label,
                        key: fieldName
                    }),
                    getFieldDecorator(fieldName, _extends({}, getFieldDecoratorOptions))((0, _fieldConfig.getFieldTypes)(fieldItem.fieldProps, fieldItem.options)[fieldItem.types])
                )
            ));
        }
    } else if (fieldItem.types !== undefined) {
        renderingFields.push(_react2.default.createElement(
            _antd.Col,
            _extends({}, fieldItem.column, { key: fieldName, style: { display: isExpand ? 'none' : 'block' } }),
            _react2.default.createElement(
                FormItem,
                _extends({}, formItemLayout, {
                    label: fieldItem.label,
                    key: fieldName
                }),
                fieldItem.types()
            )
        ));
    }
    return renderingFields;
};

var FormBuilder = function (_React$Component) {
    _inherits(FormBuilder, _React$Component);

    function FormBuilder() {
        var _temp, _this, _ret;

        _classCallCheck(this, FormBuilder);

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        return _ret = (_temp = (_this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
            formIsReady: false,
            formRule: {},
            fieldsValue: (0, _fieldConfig.buildMap)(_this.props.fieldsValue),
            expand: true
        }, _this.formMethod = {
            getFormData: function getFormData() {
                var formData = _this.props.form.getFieldsValue();
                // console.log(formData)
                return formData;
            },
            setField: function setField(data) {
                for (var key in data) {
                    var _this$props$form$setF;

                    _this.props.form.setFieldsValue((_this$props$form$setF = {}, _this$props$form$setF[key] = data[key], _this$props$form$setF));
                }
            },
            validateForm: function validateForm(fields) {
                var isError = false;
                var values = {};

                var fieldNeedValidator = [];
                if (fields !== undefined && fields.length > 0) {
                    fields.map(function (field) {
                        if (!_this.state.formRule.has(field)) {
                            fieldNeedValidator.push(field);
                        }
                    });
                }

                if (fieldNeedValidator.length > 0) {
                    errors = 'There are no API rules set for these fields: ' + fieldNeedValidator.toString();
                    return { errors: errors };
                } else {
                    _this.props.form.validateFields(fields, function (err, val) {
                        if (!err) {
                            values = val;
                        } else {
                            isError = true;
                            var new_error_obj = {};
                            for (var key in err) {
                                new_error_obj[key] = err[key].errors[0].message;
                            }
                            values = new_error_obj;
                        }
                    });
                    return { isError: isError, fields: values };
                }
            },
            resetForm: function resetForm() {
                _this.props.form.resetFields();
            },
            clearForm: function clearForm() {
                _this.props.form.resetFields();
                for (var _iterator = _this.state.fieldsValue, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
                    var _this$props$form$setF2;

                    var _ref2;

                    if (_isArray) {
                        if (_i >= _iterator.length) break;
                        _ref2 = _iterator[_i++];
                    } else {
                        _i = _iterator.next();
                        if (_i.done) break;
                        _ref2 = _i.value;
                    }

                    var _ref = _ref2;
                    var key = _ref[0];
                    var value = _ref[1];

                    var resetValue = typeof value === 'string' ? '' : typeof value === 'number' ? 0 : typeof value === 'boolean' ? false : null;
                    _this.props.form.setFieldsValue((_this$props$form$setF2 = {}, _this$props$form$setF2[key] = resetValue, _this$props$form$setF2));
                }
            }
        }, _this.toggle = function () {
            var expand = _this.state.expand;

            _this.setState({ expand: !expand });
        }, _temp), _possibleConstructorReturn(_this, _ret);
    }

    FormBuilder.prototype.componentWillMount = function componentWillMount() {
        var _this2 = this;

        var validateUrl = this.props.validateUrl.split(':');
        var method = validateUrl[0] === 'POST' ? _axiosClient.Request.POST : _axiosClient.Request.GET;
        _axiosClient.RestClient.call((0, _axiosClient.Endpoint)(method, validateUrl[1]), null, null).then(function (res) {
            // console.log(res.data)
            _this2.setState({ formRule: (0, _fieldConfig.buildMap)(res.data) });
            // console.log(this.state.formValidation)
            _this2.setState({ formIsReady: true });
        });
    };

    FormBuilder.prototype.render = function render() {
        //export inner method to be able to use from outside
        var handler = this.props.handler;

        handler(this.formMethod);

        var getFieldDecorator = this.props.form.getFieldDecorator;


        var hasExpandFields = this.props.expandFields !== undefined && Object.keys(this.props.expandFields).length !== 0 && this.props.expandFields.constructor === Object ? true : false;
        var expandSetting = this.props.expandSetting !== undefined ? this.props.expandSetting : null;

        return this.state.formIsReady ? _react2.default.createElement(
            _antd.Form,
            _extends({}, this.props.formLayout ? { layout: this.props.formLayout } : {}, { onSubmit: this.props.onSubmit }),
            renderRow(this.props, this.props.defaultFields, this.state.fieldsValue, this.state.formRule, false),
            hasExpandFields ? _react2.default.createElement(
                _antd.Row,
                { gutter: 24 },
                _react2.default.createElement(
                    _antd.Col,
                    expandSetting,
                    _react2.default.createElement(
                        'a',
                        { style: { marginLeft: 8, fontSize: 12 }, onClick: this.toggle },
                        this.state.expand ? expandSetting.expand.text : expandSetting.collapse.text,
                        this.state.expand ? expandSetting.expand.icon : expandSetting.collapse.icon
                    )
                )
            ) : null,
            hasExpandFields ? renderRow(this.props, this.props.expandFields, this.state.fieldsValue, this.state.formRule, this.state.expand) : null,
            this.props.children
        ) : _react2.default.createElement(_Loading2.default, { size: 'large' });
    };

    return FormBuilder;
}(_react2.default.Component);

exports.default = _antd.Form.create()(FormBuilder);
module.exports = exports['default'];